from os import path, mkdir
import logging
from logging.handlers import RotatingFileHandler
from pathlib import Path
import confuse

from .utils import ObjectView

from flask import Flask
from flask_bootstrap import Bootstrap
from flask_login import LoginManager

def read_config(application):
    # pylint: disable=no-member
    conf = confuse.Configuration('biscd', __name__)
    try:
        conf['logging'].get()
    except confuse.NotFoundError as not_found:
        raise ValueError("Failed to open any configfile containing a 'logging' object. " \
            "Please make sure the module directory contains a config_default.yaml, " \
            f"or /etc/{__name__}/config.yaml exists") from not_found
    conf_object = ObjectView(conf.get())
    application.config.from_object(conf_object)
    return conf

def setup_logging(application):
    # pylint: disable=no-member
    if config['logging']['log_to_stdout'].get(bool):
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(logging.INFO)
        application.logger.addHandler(stream_handler)
    else:
        logfile = Path(config['logging']['logfile'].as_filename())
        if not logfile.parent.exists():
            logfile.parent.mkdir(mode=0o770, parents=True)
        file_handler = RotatingFileHandler(str(logfile), maxBytes=10240, backupCount=10)
        file_handler.setFormatter(logging.Formatter(
            '%(asctime)s %(levelname)s: %(message)s '
            '[in %(pathname)s:%(lineno)d]'))
        file_handler.setLevel(logging.INFO)
        application.logger.addHandler(file_handler)

    application.logger.info('Biscd startup')

app = Flask(__name__)
config = read_config(app)
bootstrap = Bootstrap(app)
login = LoginManager(app)
login.login_view = 'login'

setup_logging(app)

from biscd import routes, errors, models
