from flask_wtf import FlaskForm
from wtforms import SubmitField, StringField, PasswordField, BooleanField
from wtforms.validators import DataRequired, Email, EqualTo, Length, ValidationError
from git import cmd as git_cmd, GitCommandError

from .models import User, Project

class LoginForm(FlaskForm):
    username = StringField('Username', validators=[DataRequired()])
    password = PasswordField('Password', validators=[DataRequired()])
    remember_me = BooleanField('Remember Me')
    submit = SubmitField('Sign In')

class RegistrationForm(FlaskForm):
    username = StringField('Username', validators=[DataRequired(), Length(min=4, max=64)])
    email = StringField('Email', validators=[DataRequired(), Email()])
    password = PasswordField('Password', validators=[DataRequired(), Length(min=4, max=128)])
    password2 = PasswordField(
            'Repeat Password', validators=[DataRequired(), EqualTo('password')])
    submit = SubmitField('Register')

    def validate_username(self, username):
        user = User.get(name=username.data)
        if any(user):
            raise ValidationError('Please use a different username.')

    def validate_email(self, email):
        user = User.get(email=email.data)
        if any(user):
            raise ValidationError('Please use a different email adress.')

class ProjectForm(FlaskForm):
    projectname = StringField('Project Name', validators=[DataRequired()])
    git_repo = StringField('Git Repo')
    public = BooleanField('Public Project')
    submit = SubmitField('Save')
    old_name = ''

    def validate_projectname(self, projectname):
        if projectname.data == self.old_name:
            return
        project = Project.get(name=projectname.data)
        if any(project):
            raise ValidationError('Please use a different projectname.')

    def validate_git_repo(self, git_repo):
        if not git_repo.data:
            return
        try:
            git_cmd.Git().ls_remote(git_repo.data)
        except GitCommandError as gce:
            raise ValidationError('Please enter a valid git repository url') from gce

class EmptyForm(FlaskForm):
    submit = SubmitField('Submit')
