from pathlib import Path
from werkzeug.security import generate_password_hash, check_password_hash
from flask_login import UserMixin

from biscd import config
from biscd import login
from .yaml_serializable import YamlSerializable
from .project import Project

class User(UserMixin, YamlSerializable):

    #@property
    @classmethod
    def _storage_file(cls):
        return Path(config.config_dir()) / 'users.yaml'

    #@property
    @classmethod
    def _yaml_object_name(cls):
        return 'users'

    def get_id(self):
        """Override for UserMixin"""
        return self.name

    def __init__(self, name=None, email=None):
        super().__init__()
        self.name = name
        self.email = email
        self.password_hash = None

    @property
    def projects(self):
        return set(Project.get(access__owners = self.name) + Project.get(access__public = True))

    @projects.setter
    def projects(self, projects):
        pass

    def set_password(self, password):
        self.password_hash = generate_password_hash(password)

    def check_password(self, password):
        if not password:
            return False
        return check_password_hash(self.password_hash, password)

@login.user_loader
def load_user(name):
    return User.first(name=name)
